#include <stdbool.h>
#include "io430.h"
#include "display01.h"

static const int numberOfRounds = 10;
static const int minDelay = 1000;
static const int maxDelay = 4000;

void write(unsigned int a);
int random(void);

int main(void)
{
  initialize();
  
  unsigned long sumValues = 0;
  unsigned long sumSquares = 0;
  for(int r = 0; r < numberOfRounds; r++)
  {
    int delayTime = minDelay + random() % (maxDelay - minDelay);
    waitMilliseconds(delayTime);    
    for(int i = 0; i < 8; i++)
    {
      putCharacter(0, i,'*'); 
    }
    unsigned int t = 16; // weil wir beim Schreiben auf den Bildschirm schon so lange gewartet haben
    while((P1IN & BIT3) != 0)
    {
      t++;
      waitOneMillisecond();      
    }
    clear();
    write(t);
    sumValues += t;
    sumSquares += t*(unsigned long)t; // t ins Quadrat wird zu gro fr unsigned int!
  }
  
  // Das "... + Hlfte des Nenners" sorgt fr kaufmnnische Rundung.
  // (unsigned int) msste man in C und C++ nicht davorschreiben;
  // es macht aber klar, dass der Programmierer sich bewusst ist,
  // dass Bits abgeschnitten werden knnten.
  unsigned int mean = (unsigned int)((sumValues + numberOfRounds/2) / numberOfRounds);
  
  // nochmal der Trick mit der Rundung
  unsigned long variance = (unsigned long)((sumSquares - (sumValues * sumValues + numberOfRounds / 2) / numberOfRounds + numberOfRounds/2) / (numberOfRounds - 1));
  
  unsigned int standardDeviation = 0;
  // Wurzel ziehen ohne Gleitkomma und ohne viel Nachdenken ;-)
  // Die Rundung ist mit dem + standardDeviation schon gut, aber noch nicht perfekt.
  // Vorsicht: standardDeviation ins Quadrat knnte zu gro fr unsigned int werden.
  while(standardDeviation * (unsigned long)standardDeviation + standardDeviation < variance)
  {
    standardDeviation++;
  }
    
  while(true)
  {
    waitMilliseconds(1000);
    clear();
    putCharacter(0, 0, 'M');
    write(mean);
    
    waitMilliseconds(1000);
    clear();
    putCharacter(0, 0, 'S');
    putCharacter(0, 1, 'D');    
    write(standardDeviation);
  }
    
  // berflssig:
  // return 0;
}

void write(unsigned int a)
{
  if(a == 0)
  {
    putCharacter(0, 7, '0');
  }
  else
  {
    int i = 0;
    while(a > 0)
    {
      putCharacter(0, 7-i, '0' + a % 10);
      a /= 10;
      i++;
    }
  }
}

int random(void)
{
  static unsigned long a = 42;
  a = 1103515245 * a + 12345;
  return a & 0x7FFF;  
}